/*
 * State.java
 *
 * Created on 5. listopad 2006, 9:57
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package osmctverecku;

import java.awt.Point;
import java.util.Arrays;
import java.util.Random;

/**
 *
 * @author pirko
 */
public class State {
    
    private int[][] field = new int[3][3];
    
    private static final int[][] solution = {
        {1,2,3},
        {8,0,4},
        {7,6,5}};
    
    private Point zeroPos;
    
    public static final State genState() {
        Random random = new Random();
        State state = new State(solution,new Point(1,1));
        for (int i=0;i<random.nextInt(5000);i++) {
            switch(random.nextInt(4)) {
                case 0:
                    State n = state.up();
                    if (n != null) state = n;
                    break;
                case 1:
                    State n1 = state.down();
                    if (n1 != null) state = n1;
                    break;
                case 2:
                    State n2 = state.left();
                    if (n2 != null) state = n2;
                    break;
                case 3:
                    State n3 = state.right();
                    if (n3 != null) state = n3;
                    break;
                   
            }
        } 
        return state;
    }
        
    public State(int[][] field,Point zeroPos) {
        this.field = field;
        this.zeroPos = zeroPos;
    }
    
    /**
     * @returns null if not possible
     */
    public State up() {
        if (zeroPos.y-1 < 0)
            return null;
        
        int[][] newField = copyField(field);
        Point newZero = new Point(zeroPos.x,zeroPos.y-1);
        newField[zeroPos.y][zeroPos.x] = field[newZero.y][newZero.x];
        newField[newZero.y][newZero.x] = field[zeroPos.y][zeroPos.x];
        
        return new State(newField,newZero);
    }
    /**
     * @returns null if not possible
     */
    public State down() {
        if (zeroPos.y+1 > 2)
            return null;
        
        int[][] newField = copyField(field);
        Point newZero = new Point(zeroPos.x,zeroPos.y+1);
        newField[zeroPos.y][zeroPos.x] = field[newZero.y][newZero.x];
        newField[newZero.y][newZero.x] = field[zeroPos.y][zeroPos.x];
        
        return new State(newField,newZero);
    }
    /**
     * @returns null if not possible
     */
    public State left() {
        if (zeroPos.x-1 < 0)
            return null;
        
        int[][] newField = copyField(field);
        Point newZero = new Point(zeroPos.x-1,zeroPos.y);
        newField[zeroPos.y][zeroPos.x] = field[newZero.y][newZero.x];
        newField[newZero.y][newZero.x] = field[zeroPos.y][zeroPos.x];
        
        return new State(newField,newZero);
    }
    /**
     * @returns null if not possible
     */
    public State right() {
        if (zeroPos.x+1 > 2)
            return null;
        
        int[][] newField = copyField(field);
        Point newZero = new Point(zeroPos.x+1,zeroPos.y);
        newField[zeroPos.y][zeroPos.x] = field[newZero.y][newZero.x];
        newField[newZero.y][newZero.x] = field[zeroPos.y][zeroPos.x];
        
        return new State(newField,newZero);
    }
    
    /**
     * first heuristik
     */
    public int badCount() {
        int res = 0;
        for (int i=0;i<3;i++)
            for (int j=0;j<3;j++) {
                if (field[i][j] != solution[i][j] && field[i][j] != 0) {
                    res++;
                }
            }
        
        return res;
    }
    
    /**
     * second heurstik
     */
    public int manhattan() {
        int res = 0;
        for (int i=0;i<3;i++)
            for (int j=0;j<3;j++) {
                switch (field[i][j]) {
                    case 0:
                        break;
                    case 1:
                        res += i+j;
                        break;
                    case 2:
                        res += i+Math.abs(j-1);
                        break;
                    case 3:
                        res += i+Math.abs(j-2);
                        break;
                    case 4:
                        res += Math.abs(i-1)+Math.abs(j-2);
                        break;
                    case 5:
                        res += Math.abs(i-2)+Math.abs(j-2);
                        break;
                    case 6:
                        res += Math.abs(i-2)+Math.abs(j-1);
                        break;
                    case 7:
                        res += Math.abs(i-2)+j;
                        break;
                    case 8:
                        res += Math.abs(i-1)+j;
                        break;
                }
            }
        
        return res;
    }
    
    public boolean isSolution() {
        return badCount() == 0;
    }
    
    
    /**
     *Carefukl copys only first 3x3 matrix
     */
    private static int[][] copyField(int[][] field) {
        int[][] res = new int[3][3];
        for (int i=0;i<3;i++)
            for (int j=0;j<3;j++)
                res[i][j] = field[i][j];
        return res;
        
    }
    
    
    public void print() {
        StringBuffer buf = new StringBuffer();
        for (int i=0;i<3;i++) {
            buf.append("|");
            for (int j=0;j<3;j++)
                buf.append(String.valueOf(field[i][j]));
            buf.append("|\n");
        }
        
        System.out.println(buf.toString());
        
    }

    public boolean equals(Object obj) {
        if (!(obj instanceof  State))
            return false;
        return Arrays.deepEquals(field,((State)obj).field);
    }
    
    
    
}
